@@ -89,14 +89,18 @@ void IALr(double* y, double* RX, double* bInit, double* RbSample, double* Rb0,
   double* tau2use)
 {
   double **X;
+  int **IDC;
+
   int n = dims[0];
   int p = dims[1];
-  
+  int Times_cv = dims[12];
+
   /* reorganize vector into matrix */
+  reorg_int(Ridc, &IDC, n, Times_cv);
   reorg(RX, &X, n, p);
 
   IAL(y, X, bInit, RbSample, Rb0, Rdelta, Rtau, dims, Repsilon, nIter, 
-    b_update_order, Rscore, RscoNA, RBIC_gamma, Ridc, Rcriterion, 
+    b_update_order, Rscore, RscoNA, RBIC_gamma, IDC, Rcriterion, 
     score2use, delta2use, tau2use);
 }
 
@@ -115,13 +119,18 @@ void IALc(double* y, char** fname, double* bInit, double* RbSample, double* Rb0,
   double* tau2use)
 {
   double **X;
+  int ** IDC;
   int i;
   int n = dims[0];
   int p = dims[1];
   int offsetrow  = dims[5];
   int offsetcol  = dims[6];
   int transposeX = dims[7];
-  
+  int Times_cv   = dims[12];
+
+  /* reorganize vector into matrix */
+  reorg_int(Ridc, &IDC, n, Times_cv);
+
   /* allocate memory */
   X = (double **) malloc(n * sizeof(double*));
   X[0] = (double *) calloc(n*p, sizeof(double));
@@ -136,7 +145,7 @@ void IALc(double* y, char** fname, double* bInit, double* RbSample, double* Rb0,
   readtext(X, fname[0], n, p, offsetrow, offsetcol, transposeX);
 
   IAL(y, X, bInit, RbSample, Rb0, Rdelta, Rtau, dims, Repsilon, nIter, 
-    b_update_order, Rscore, RscoNA, RBIC_gamma, Ridc, Rcriterion,
+    b_update_order, Rscore, RscoNA, RBIC_gamma, IDC, Rcriterion,
     score2use, delta2use, tau2use);
   
   free(X[0]);
@@ -154,14 +163,16 @@ void IALc(double* y, char** fname, double* bInit, double* RbSample, double* Rb0,
 void IAL(double* y, double** X, double* bInit, double* RbSample, double* Rb0,
   double* Rdelta, double* Rtau, int* dims, double* Repsilon, int* nIter, 
   int* b_update_order, double* Rscore, int* RscoNA, double* RBIC_gamma, 
-  int* Ridc, char** Rcriterion, double* score2use, double* delta2use, 
+  int** IDC, char** Rcriterion, double* score2use, double* delta2use, 
   double* tau2use)
 {
-  int i, j, jk, k, c1, w, n, p, L, init_b, *Jset, n_b2use, *w_b2use;
+  int i, j, jk, k, c1, t1, w, n, p, L, init_b, *Jset, n_b2use, *w_b2use;
+  int s1=0, Times_cv, total;
   int Nmax, found=0, n_delta, n_tau, p_max, k1, k2, ncv, **scoNA;
   int *i_test, nTrain, nTest;
   double xij, b0, bj_bar, v0, sj, tmp, epsilon, tau1, delta1, delta2;
-  double *b, *b1, *v, *res, *Xj2, **score, *rssCV, rss1;
+  double diff=0.0, tsumrescv;
+  double *b, *b1, *v, *res, *Xj2, **score, *rssCV, rss1, *rescv;
   double *y_test, **X_train, *y_train;
   double mse1, BIC_gamma = *RBIC_gamma;
   char* criterion = Rcriterion[0];
@@ -172,13 +183,15 @@ void IAL(double* y, double** X, double* bInit, double* RbSample, double* Rb0,
   p = dims[1];
   L = dims[2];
   Nmax = dims[3];
-  init_b  = dims[4];
-  n_delta = dims[8];
-  n_tau   = dims[9];
-  p_max   = dims[10];
-  ncv     = dims[11];
-  epsilon = *Repsilon;
-  
+  init_b   = dims[4];
+  n_delta  = dims[8];
+  n_tau    = dims[9];
+  p_max    = dims[10];
+  ncv      = dims[11];
+  Times_cv = dims[12];
+  epsilon  = *Repsilon;
+  total    = ncv*Times_cv;
+
   // socre can be eitehr BIC or rss/n for CV
   *score2use = DBL_MAX;
   
@@ -188,7 +201,7 @@ void IAL(double* y, double** X, double* bInit, double* RbSample, double* Rb0,
 
   /* allocate memory */
   w_b2use = (int *)calloc(p_max, sizeof(int));
-  rssCV   = (double *)calloc(ncv, sizeof(double));
+  rssCV   = (double *)calloc(ncv*Times_cv, sizeof(double));
 
   if(strcmp(criterion, "cv")==0){
     y_train = (double *)calloc(n, sizeof(double));
@@ -197,12 +210,13 @@ void IAL(double* y, double** X, double* bInit, double* RbSample, double* Rb0,
     i_test  = (int *)calloc(n, sizeof(int));
   }
   
-  b    = (double *)calloc(p, sizeof(double));
-  b1   = (double *)calloc(p, sizeof(double));
-  v    = (double *)calloc(p, sizeof(double));
-  res  = (double *)calloc(n, sizeof(double));
-  Xj2  = (double *)calloc(p, sizeof(double));
-  Jset = (int *)calloc(p, sizeof(int));
+  b     = (double *)calloc(p, sizeof(double));
+  b1    = (double *)calloc(p, sizeof(double));
+  v     = (double *)calloc(p, sizeof(double));
+  res   = (double *)calloc(n, sizeof(double));
+  Xj2   = (double *)calloc(p, sizeof(double));
+  Jset  = (int *)calloc(p, sizeof(int));
+  rescv = (double *)calloc(n, sizeof(double));
 
   for(j=0; j<p; j++){
     Jset[j] = j;
@@ -246,8 +260,7 @@ void IAL(double* y, double** X, double* bInit, double* RbSample, double* Rb0,
         // Rprintf("k1=%d, k2=%d, delta1=%f, tau1=%f\n", k1, k2, delta1, tau1);
 
         b0 = 0.0;
-        v0 = var(y, n);
-        
+
         if(init_b){
           for(j=0; j<p; j++){
             b[j] = b1[j] = bInit[j];
@@ -266,18 +279,21 @@ void IAL(double* y, double** X, double* bInit, double* RbSample, double* Rb0,
         /* initialize residuals */
         for(i=0; i<n; i++){
           res[i] = y[i];
+          for(j=0; j<p; j++){
+            res[i] -= X[i][j]*b[j];
+          }
         }
         
+        b0 = mean(res, n);
+        
+        for(i=0; i<n; i++){
+          res[i] -= b0;
+        }
+        
+        v0 = var(res, n);
+
         for(w=1; w<=Nmax; w++){
-          /**
-           * step 2. Update b0
-           */
-          b0 = mean(res, n);
-      
-          for(i=0; i<n; i++){
-            res[i] -= b0;
-          }
-      
+                    
           /**
            * step 3.1 choose the order of updating b[j]
            */
@@ -318,6 +334,19 @@ void IAL(double* y, double** X, double* bInit, double* RbSample, double* Rb0,
             }
           }
       
+          /**
+           * step 2. Update b0 and res
+           */
+          
+          diff = mean(res, n);
+          
+          for(i=0; i<n; i++){
+            res[i] = res[i] - diff;
+          }
+          
+          b0 = b0 + diff;
+          
+          
           /**
            * step 4 Update v0, i.e. sigma^2
            */
@@ -326,15 +355,15 @@ void IAL(double* y, double** X, double* bInit, double* RbSample, double* Rb0,
             v0 += res[i]*res[i];
           }
           v0 /= n;
-      
+          
           /**
            * step 5 Update v[j], i.e. kappa_j
            */
           // Rprintf("w=%d, tau=%e, delta=%e\n", w, tau, delta);
           for(j=0; j<p; j++){
             v[j] = (fabs(b[j]) + tau1)/delta2;
-          }
-      
+          }          
+          
           /**
            * check convergence
            */          
@@ -396,185 +425,213 @@ void IAL(double* y, double** X, double* bInit, double* RbSample, double* Rb0,
        **********************************************************/
       }else if(strcmp(criterion, "cv")==0){
       
-        for(c1=0; c1 < ncv; c1++){
+        s1=0;
         
-          /**
-           * step 1.1 Initialization
-           */
-          b0 = 0.0;
-          
-          if(init_b){
-            for(j=0; j<p; j++){
-              b[j] = b1[j] = bInit[j];
-              v[j] = (fabs(b[j]) + tau1)/delta2;
-            }    
-          }else{
-            for(j=0; j<p; j++){
-              b[j] = b1[j] = 0.0;
-              v[j] = tau1/delta2;
-            }
-          }
-          
-          /**
-           * step 1.2 Split the data
-           */
+        for (t1=0; t1 < Times_cv; t1++){
 
-          k = 0;
-          nTrain = 0;
-          nTest  = 0;
-          for(i=0; i<n; i++){
-            if(Ridc[i]!=c1){
-              X_train[nTrain] = X[i];
-              y_train[nTrain] = y[i];
-              nTrain++;
+          for(c1=0; c1 < ncv; c1++){
+        
+            /**
+             * step 1.1 Initialization
+             */
+            b0 = 0.0;
+            
+            if(init_b){
+              for(j=0; j<p; j++){
+                b[j] = b1[j] = bInit[j];
+                v[j] = (fabs(b[j]) + tau1)/delta2;
+              }    
             }else{
-              i_test[nTest] = i;
-              y_test[nTest] = y[i];
-              nTest++;
-            }
-          }
-          if(nTest + nTrain != n){
-            error("nTest + nTrain != n: %d + %d != %d\n", nTest, nTrain, n);
-          }
-          
-          for(j=0; j<p; j++){
-            Xj2[j] = 0.0;
-            for(i=0; i<nTrain; i++){
-              xij = X_train[i][j];
-              Xj2[j] += xij*xij;
-            }
-          }
-
-          /* initialize residuals */
-          for(i=0; i<nTrain; i++){
-            res[i] = y_train[i];
-          }
-
-          v0 = var(y_train, n);
-
-          for(w=1; w<=Nmax; w++){
-            /**
-             * step 2. Update b0
-             */
-            b0 = mean(res, nTrain);
-        
-            for(i=0; i<nTrain; i++){
-              res[i] -= b0;
-            }
-        
-            /**
-             * step 3.1 choose the order of updating b[j]
-             */
-            if(*b_update_order > 1)
-            get_b_order(*b_update_order, Jset, w, nTrain, p, y_train,X_train,b);
-        
-            /**
-             * step 3.2 Update b[j]
-             */
-            for(jk=0; jk<p; jk++){
-              j = Jset[jk];
-        
-              /* remove the effect of Xj from the residual*/
-              for(i=0; i<nTrain; i++){
-                res[i] = res[i] + X_train[i][j]*b[j];
+              for(j=0; j<p; j++){
+                b[j] = b1[j] = 0.0;
+                v[j] = tau1/delta2;
               }
-        
-              bj_bar = 0.0;
-              for(i=0; i<nTrain; i++){
-                bj_bar += X_train[i][j]*res[i];
-              }
-              bj_bar /= (Xj2[j]);
-              sj   = v0/(Xj2[j]);
-              tmp  = sj/v[j];
-              if(bj_bar > tmp){
-                b[j] = bj_bar - tmp;
-              }else if(bj_bar < -tmp){
-                b[j] = bj_bar + tmp;
+            }
+            
+            /**
+             * step 1.2 Split the data
+             */
+
+            k = 0;
+            nTrain = 0;
+            nTest  = 0;
+              
+            for(i=0; i<n; i++){
+              if(IDC[i][t1]!=c1){
+                X_train[nTrain] = X[i];
+                y_train[nTrain] = y[i];
+                nTrain++;
               }else{
-                b[j] = 0.0;
+                i_test[nTest] = i;
+                y_test[nTest] = y[i];
+                nTest++;
               }
-        
-              /* add the effect of Xj back into the residual */
+            }
+              
+            if(nTest + nTrain != n){
+              error("nTest + nTrain != n: %d + %d != %d\n", nTest, nTrain, n);
+            }
+            
+            for(j=0; j<p; j++){
+              Xj2[j] = 0.0;
               for(i=0; i<nTrain; i++){
-                res[i] = res[i] - X_train[i][j]*b[j];
+                xij = X_train[i][j];
+                Xj2[j] += xij*xij;
               }
             }
-        
-            /**
-             * step 4 Update v0, i.e. sigma^2
-             */
-            v0 = 0.0;
+
+            /* initialize residuals */
+            b0 = mean(y_train, nTrain);
+
             for(i=0; i<nTrain; i++){
-              v0 += res[i]*res[i];
+              rescv[i] = y_train[i] - b0;
             }
-            v0 /= nTrain;
-        
-            /**
-             * step 5 Update v[j], i.e. kappa_j
-             */
-        
-            // Rprintf("w=%d, tau=%e, delta=%e\n", w, tau, delta);
-            for(j=0; j<p; j++){
-              v[j] = (fabs(b[j]) + tau1)/delta2;
-            }
-        
-            /**
-             * check convergence
-             */
-            
-            found = 0;
-            for(j=0; j<p; j++){
-              if(fabs(b1[j] - b[j]) > epsilon){
-                found = 1;
-                k = 0;
-                break;
+
+            v0 = var(y_train, nTrain);
+
+            for(w=1; w<=Nmax; w++){
+          
+              /**
+               * step 3.1 choose the order of updating b[j]
+               */
+              if(*b_update_order > 1)
+              get_b_order(*b_update_order, Jset, w, nTrain, p, y_train, X_train, b);
+          
+              /**
+               * step 3.2 Update b[j]
+               */
+              for(jk=0; jk<p; jk++){
+                j = Jset[jk];
+          
+                /* remove the effect of Xj from the residual*/
+                for(i=0; i<nTrain; i++){
+                  rescv[i] = rescv[i] + X_train[i][j]*b[j];
+                }
+          
+                bj_bar = 0.0;
+                for(i=0; i<nTrain; i++){
+                  bj_bar += X_train[i][j]*rescv[i];
+                }
+                
+                bj_bar /= (Xj2[j]);
+                sj      = v0/(Xj2[j]);
+                tmp     = sj/v[j];
+                
+                if(bj_bar > tmp){
+                  b[j] = bj_bar - tmp;
+                }else if(bj_bar < -tmp){
+                  b[j] = bj_bar + tmp;
+                }else{
+                  b[j] = 0.0;
+                }
+          
+                /* add the effect of Xj back into the residual */
+                for(i=0; i<nTrain; i++){
+                  rescv[i] = rescv[i] - X_train[i][j]*b[j];
+                }
+              }
+          
+              tsumrescv = 0;
+              for (i=0; i<nTrain; i++){
+                tsumrescv = tsumrescv + rescv[i];
+              }
+              diff = tsumrescv / nTrain;
+
+              for(i=0; i<nTrain; i++){
+                rescv[i] = rescv[i] - diff;
+              }
+              
+              b0 = b0 + diff;   
+              
+              /**
+               * step 4 Update v0, i.e. sigma^2
+               */
+              v0 = 0.0;
+              for(i=0; i<nTrain; i++){
+                v0 += rescv[i]*rescv[i];
+              }
+              v0 /= nTrain;
+          
+              /**
+               * step 5 Update v[j], i.e. kappa_j
+               */
+          
+              // Rprintf("w=%d, tau=%e, delta=%e\n", w, tau, delta);
+              for(j=0; j<p; j++){
+                v[j] = (fabs(b[j]) + tau1)/delta2;
+              }
+          
+              /**
+               * check convergence
+               */
+              
+              found = 0;
+              for(j=0; j<p; j++){
+                if(fabs(b1[j] - b[j]) > epsilon){
+                  found = 1;
+                  k = 0;
+                  break;
+                }
+              }
+              
+              for(j=0; j<p; j++){
+                b1[j] = b[j];
+              }
+          
+              if(found == 0){
+                k += 1;
+                if(k>=L){
+                  break;
+                }
               }
             }
             
-            for(j=0; j<p; j++){
-              b1[j] = b[j];
-            }
-        
-            if(found == 0){
-              k += 1;
-              if(k>=L){
-                break;
+            n_b2use = 0;
+            for(j=0; j < p; j++){
+              if(fabs(b1[j]) > 1e-10){
+                w_b2use[n_b2use] = j;
+                n_b2use++;
+                if(n_b2use >= p_max){
+                  break;
+                }
               }
             }
-          }
-          
-          n_b2use = 0;
-          for(j=0; j < p; j++){
-            if(fabs(b1[j]) > 1e-10){
-              w_b2use[n_b2use] = j;
-              n_b2use++;
-              if(n_b2use >= p_max){
-                break;
+            
+            /***
+             * if no covarite or too many covariates are selected, 
+             * just record the rss as the rss of y_test
+             */
+            if(n_b2use == 0 || n_b2use >= p_max){
+              rss1 = 0.0;
+              for(i=0; i < nTest; i++){
+                tmp = mean(y_train,nTrain);              
+                tmp  = y_test[i] -  tmp;
+                rss1 += tmp*tmp;
               }
-            }
-          }
-          
-          /***
-           * if no covarite or too many covariates are selected, 
-           * just record the rss as the rss of y_test
-           */
-          if(n_b2use == 0 || n_b2use >= p_max){
-            rssCV[c1] = var(y_test, nTest);
-          }else{
-            rss1 = 0.0;
-            for(i=0; i < nTest; i++){
-              tmp = 0.0;
-              for(j=0; j<n_b2use; j++){
-                tmp += X[i_test[i]][w_b2use[j]]*b[w_b2use[j]];
+              
+              rssCV[s1] = rss1/nTest;
+              s1 = s1 +1;
+            }else{
+              rss1 = 0.0;
+              for(i=0; i < nTest; i++){
+                tmp = 0.0;
+                for(j=0; j<n_b2use; j++){
+                  tmp += X[i_test[i]][w_b2use[j]]*b[w_b2use[j]];
+                }
+                tmp  = y_test[i] - b0 - tmp;
+                rss1 += tmp*tmp;
               }
-              tmp  = y_test[i] - b0 - tmp;
-              rss1 += tmp*tmp;
+              rssCV[s1] = rss1/nTest;
+              s1 += 1;
             }
-            rssCV[c1] = rss1/nTest;
           }
         }
-        
-        tmp = mean(rssCV, ncv);
+
+        if(s1 > total){
+          error("s1 > total: %d > %d\n", s1, total);
+        }
+          
+        tmp = mean(rssCV, total);
         
         if(tmp < *score2use){
           *score2use = tmp;
@@ -599,8 +656,8 @@ void IAL(double* y, double** X, double* bInit, double* RbSample, double* Rb0,
    * cross-validation
    *
    * if the criterion is cross-validation, we only selectedd 
-   * delta and tau by cv, but need to compute b0 and b
-   * using all the data again
+   * delta and tau by cv in previous code, now we need to 
+   * compute b0 and b using all the data again
    **********************************************************/
   if(strcmp(criterion, "cv")==0){
     delta1 = *delta2use;
@@ -608,6 +665,21 @@ void IAL(double* y, double** X, double* bInit, double* RbSample, double* Rb0,
     delta2 = delta1 + 1;
     
     // Rprintf("\n\ndelta=%f, tau=%f\n", delta1, tau1);
+    
+    /**
+     * Calculate Xj2
+     * sum square for each marker, i.e., each column of X
+     * note this quantity is re-calculated if we use cv to 
+     * select delta and tau
+     */
+    for(j=0; j<p; j++){
+      Xj2[j] = 0.0;
+      for(i=0; i<n; i++){
+        xij = X[i][j];
+        Xj2[j] += xij*xij;
+      }
+    }
+    
     /**
      * step 1. Initialization
      */
@@ -736,6 +808,7 @@ void IAL(double* y, double** X, double* bInit, double* RbSample, double* Rb0,
   PutRNGstate();
   
   free(b);
+  free(b1);
   free(v);
   free(res);
   free(Xj2);
