@@ -5,7 +5,7 @@ void get_b_order(int b_update_order, int* Jset, int w, int n, int p,
 void IAL(double* y, double** X, double* bInit, double* RbSample, double* Rb0,
   double* Rdelta, double* Rtau, int* dims, double* Repsilon, int* nIter, 
   int* b_update_order, double* Rscore, int* RscoNA, double* RBIC_gamma, 
-  int* Ridc, char** Rcriterion, double* score2use, double* delta2use, 
+  int** IDC, char** Rcriterion, double* score2use, double* delta2use, 
   double* tau2use);
       
 void bayes_AL(double* y, double* RX, double* bInit, double* RbSample, 
